import serial
import datetime
import asyncio
import websockets
import sys

# Ajuste para compatibilidade com o Windows
if sys.platform == 'win32':
    asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())

# Configurar a porta serial
try:
    ser = serial.Serial('COM1', 9600, timeout=0)
except:
    ser = serial.Serial('COM2', 9600, timeout=0)

# Função para escrever no log
def log_data(data):
    with open('log.txt', 'a') as f:
        timestamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        f.write(f'{timestamp}: {data}\n')

# Função para enviar dados via WebSocket
async def send_via_websocket(websocket, path):
    try:
        while True:
            if ser.in_waiting:
                data = ser.readline().decode('latin-1', errors='replace').strip()
                if data:
                    data = data[0]
                    print(f'Dados recebidos: {data}')
                    log_data(data)
                    await websocket.send(data)  # Envia o dado via WebSocket
            await asyncio.sleep(0)  # Permite que outras tarefas sejam processadas
    except Exception as e:
        print(f"Ocorreu um erro: {e}")
    finally:
        #ser.close()
        print("Limpando buffer")
        ser.reset_input_buffer()

# Iniciar o servidor WebSocket
async def websocket_server():
    async with websockets.serve(send_via_websocket, "localhost", 8765):
        await asyncio.Future()  # Mantém o servidor rodando indefinidamente

# Iniciar o loop principal
try:
    asyncio.run(websocket_server())
except KeyboardInterrupt:
    print("Servidor WebSocket encerrado pelo usuário.")
    ser.close()
except Exception: 
    print('Houve um erro')
    print(Exception)
    ser.reset_input_buffer()