<?php
namespace Service;

use Exception;
use LDAP\Result;
use \Remote\Printer\Command;

class Impressao {

	protected $entityManager;
	protected $util;
	protected $impressaoUtil;
	protected $instanceManager;
	private $getOrigins;
	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Util\Util $util,
		\Lib\ImpressaoUtil $impressaoUtil,
		\Service\GetOrigins $getOrigins,
		\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager
	){
		$this->entityManager		= $entityManager;
		$this->util					= $util;
		$this->impressaoUtil		= $impressaoUtil;
		$this->getOrigins			= $getOrigins;
		$this->instanceManager = $instanceManager;
	}

	public function imprimePicklist($produtos, $IDIMPPRODAPETIC = null){
		$produtos = $this->getInformacoesProdutos($produtos);
		$params = $this->getParamsProdutos($produtos);
		$MODALIDADE = $this->getOrigins->manageOrigin($params['IDORIGEMVENDA'], "print", "modality");
		$ORIGEM = $this->getOrigins->manageOrigin($params['IDORIGEMVENDA'], "print", "origin");
		$CDCAIXA =  $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_CDCAIXA_BY_CDSETOR, array('CDSETOR' => $produtos[0]['CDSETOR']));
		$GARCOM = '';
		if ($MODALIDADE === 'Mesa' && isset($params['NRVENDAREST']) && $params['NRVENDAREST']) {
			$result = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_VENDEDOR, array('NRVENDAREST'=> $params['NRVENDAREST']));
			if ($result && $result['NMFANVEN']) {
				$GARCOM = $result['NMFANVEN'];
			}
		}
		$CDFILIAL = $params['CDFILIAL'];
		$CDCAIXA = $CDCAIXA['CDCAIXA'];
		// $NMCAIXA = $params['NMCAIXA'];
		// $NRPEDIDOFOS = $params['NRPEDIDOFOS'];
		$SENHA = $params['CDSENHAPED'];
		$NRCOMANDA = $params['NRCOMANDA'];
		$DTHORA = $params['DTHORA'];
		$CDLOJA = $params['CDLOJA'];
		$NRVENDAREST = $params['NRVENDAREST'];
		$NRSEQVENDA = $params['NRSEQVENDA'];

		$dadosImpressora = $this->impressaoUtil->getDadosImpressora($CDFILIAL, $CDCAIXA);
	    if (!$dadosImpressora['error'] && $dadosImpressora['dadosImpressora']){
			$comandos = new Command();
			$dadosImpressora = $dadosImpressora['dadosImpressora'];
			$paramsImp = array(
				':CDFILIAL'			=>	$CDFILIAL,
				':CDLOJA'			=>	$CDLOJA,
				':CDCAIXA'			=>	$CDCAIXA
			);

			$r_valImprLoja = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_VAL_IMPR_LOJA, $paramsImp);
			if (!$r_valImprLoja || empty($r_valImprLoja)) {
				$result['error'] = true;
	    		$result['message'] = 'Não foi possivel obter os dados da impressora.';
				return $result;
			}
			$IDMODEIMPRES = $r_valImprLoja['IDMODEIMPRES'];

			$printerParams = $this->impressaoUtil->buscaParametrosImpressora($IDMODEIMPRES);

			$printerParams['letterType'] = $printerParams['tipoLetra'];
			$text = '';
			$text .= $this->impressaoUtil->imprimeLinha($printerParams);
			
			// verifica se é para imprimir o número da mesa e a posição
			$imprimeMesaPosicao = $this->instanceManager->getParameter("printTablePositionPicklist");
			$topText = $MODALIDADE;
			if ($imprimeMesaPosicao) {
				if (isset($produtos[0])) {
					$MESA = isset($produtos[0]['NRMESA']) ? $produtos[0]['NRMESA'] : "";
					// $text .= $this->impressaoUtil->centraliza($printerParams, chr(27).chr(33).chr(48).$MODALIDADE . ' - ' . $MESA.chr(27).chr(33), $printerParams['largura']).$printerParams['comandoEnter'];
					if ($MESA != "") {
						$topText .= ' - ' . $MESA;
					}
				}
			} 
			$text .= $this->impressaoUtil->centraliza($printerParams, chr(27).chr(33).chr(48).$topText.chr(27).chr(33), $printerParams['largura']).$printerParams['comandoEnter'];
			//colocar string grande e invertida (fundo preto)
			$text .= $this->impressaoUtil->imprimeLinha($printerParams);
			$text .= $printerParams['comandoEnter'];
			$printerParams['bold'] = true;

			$comandos->text($text, $printerParams);
			//abaixo normal
			$text = '';
			$printerParams['bold'] = false;
			$text .= $this->impressaoUtil->separaComEspacos('Linha 1', $ORIGEM, $printerParams['largura']);

			$text .=  $printerParams['comandoEnter'] . $printerParams['comandoEnter'];

			// $text .= 'Loja   : ' . $CDLOJA.' - '.$NMLOJA . $printerParams['comandoEnter'];
			// $text .= 'Caixa  : ' . $CDCAIXA.' - '.$NMCAIXA . $printerParams['comandoEnter'];

			$comandos->text($text, $printerParams);
			//colocar string grande e invertida (fundo preto)
			$text = '';
			if($SENHA != null){
				$printerParams['bold'] = false;
				$senhaNegrito = chr(27).chr(33).chr(48).$SENHA.chr(27).chr(33); //coloca a senha em negrito e extended, e o texto após a senha normal
				$text .= 'Pedido: ' . $senhaNegrito . $printerParams['comandoEnter'] . $printerParams['comandoEnter'] . substr($DTHORA, 11, 20);
			} else{
				$text .= $this->impressaoUtil->alinhaDireita(substr($DTHORA, 11, 20),  $printerParams['largura']);
			}

			$text .=  $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
			$comandos->text($text, $printerParams);

			$text = '';
			$printerParams['bold'] = false;
			// produtos em texto normal
			$text .= $this->impressaoUtil->alinhaEsquerda('Produtos  :',  $printerParams['largura']);
			$text .=  $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
			$comandos->text($text, $printerParams);

			// nome do produto em negrito e expanded
			$text = chr(27).chr(33).chr(48);
			//$text .= chr(27).chr(77).chr(1);
			$printerParams['italic'] = false;
			$printerParams['bold'] = false;
			$printerParams['expanded'] = false;

			$name = array_column($produtos, 'NMPRODUTO');
			array_multisort($name, SORT_ASC, $produtos);
			
			// ordenar pela ordem da expedicao caso ordenaProdExp seja true
			// a ordem da expedicao é ordem alfabetica agrupada por posicao, entao ordena por nome (feito acima) e depois por posicao (aqui)
			$ordenaProdExp = $this->instanceManager->getParameter("printPicklistExpOrder");
			if ($ordenaProdExp) {
				// NRLUGARMESAIT ou DSLUGARMESAIT
				if (isset($produtos[0]) && isset($produtos[0]['NRLUGARMESAIT'])) {
					$position = array_column($produtos, 'NRLUGARMESAIT');
					array_multisort($position, SORT_ASC, $produtos);
				} else if (isset($produtos[0]) && isset($produtos[0]['DSLUGARMESAIT'])) {
					$position = array_column($produtos, 'DSLUGARMESAIT');
					array_multisort($position, SORT_ASC, $produtos);
				}
			}

			$currentPosition = "";

			foreach ($produtos as $key => &$produto) {
				if ($imprimeMesaPosicao && isset($produto['NRMESA']) && isset($produto['NRLUGARMESAIT'])) {
					$prodPosition = isset($produto['NRLUGARMESAIT']) && $produto['NRLUGARMESAIT'] ? $produto['NRLUGARMESAIT'] : "";

					if ($prodPosition == "") {
						$prodPosition = isset($produto['DSLUGARMESAIT']) ? explode(" ", $produto['DSLUGARMESAIT'])[1] : "";
					}

					if ($prodPosition != $currentPosition) {
						$currentPosition = $prodPosition;

						$positionText = $prodPosition != "" ? "Posicao " . $prodPosition : "";

						//tira o negrito e expanded para imprimir a posição
						$text .= chr(27).chr(33).chr(10);
						$text .= $this->impressaoUtil->alinhaEsquerda($positionText,  $printerParams['largura']). $printerParams['comandoEnter'];
						// coloca o negrito de volta
						$text .= chr(27).chr(33).chr(48);
					}
				}

				$prodQuantName = $produto['QTPRODPEFOS'] . 'x ' . $produto['NMPRODUTO'];
				$text .= $this->impressaoUtil->alinhaEsquerda($prodQuantName ,  $printerParams['largura']). $printerParams['comandoEnter'];

				//se o TXPRODPED chegar como string, converte em array
				if(!is_array($produto['TXPRODPED']) && !empty($produto['TXPRODPED'])){
					$decodedTxprodped = json_decode($produto['TXPRODPED']);
					$obsArray = array();
					foreach($decodedTxprodped as $current) {
						array_push($obsArray, get_object_vars($current));
					}

					$produto['TXPRODPED'] = $obsArray;
				}

				//se a observação do TXPRODPED não tiver apelido, coloca o nome da observação
				$observacoesTexto = "";
				$obsName = array();
				if ($produto['TXPRODPED'] != null) {
					foreach($produto['TXPRODPED'] as $key => &$obs){
						if(empty($obs['DSAPELIDOOBS']) || $obs['DSAPELIDOOBS'] == null) {
							$obs['DSAPELIDOOBS'] = $obs['OBSERVACAO'];
						}
						$obs['IDCONTROLAOBS'] = "";
						if(stripos($obs['DSAPELIDOOBS'], "-") !== false){
							$obs['IDCONTROLAOBS'] = '-';
							$obs['DSAPELIDOOBS'] = str_replace("-", "", $obs['DSAPELIDOOBS']);
						};
						if(stripos($obs['DSAPELIDOOBS'], "+") !== false){
							$obs['IDCONTROLAOBS'] = '+';
							$obs['DSAPELIDOOBS'] = str_replace("+", "", $obs['DSAPELIDOOBS']);
						};
					}

					// ordena TXPRODPED
					$name = array_column($produto['TXPRODPED'], 'DSAPELIDOOBS');
					array_multisort($name, SORT_ASC, SORT_STRING | SORT_FLAG_CASE, $produto['TXPRODPED']);

					if($produto['TXPRODPED'] != null) {
						// array_column($produto['TXPRODPED'], 'DSAPELIDOOBS'); // pega os apelidos das observacoes para imprimir no qrCode
						foreach ($produto['TXPRODPED'] as $key => &$obs) {
							array_push($obsName, $obs['IDCONTROLAOBS'] . $obs['DSAPELIDOOBS']);
						}
						$observacoesTexto = implode(" - ", $obsName); // separa as observacoes por '-'
					}
				}
				$produto['observacoesArray'] = $obsName;
				$produto['observacoesTexto'] = $observacoesTexto;
			}

			if ($GARCOM) {
				$text.= $printerParams['comandoEnter'];
				$text .= $this->impressaoUtil->centraliza($printerParams, chr(27).chr(33).chr(48).'GARÇOM: '.$GARCOM, $printerParams['largura']).$printerParams['comandoEnter'];
			}
			// $comandos->text($text, $printerParams);

			//$text .= '1B+21+0';
			$text .= $printerParams['comandoEnter'] . $printerParams['comandoEnter'];

	        $comandos->text($text, $printerParams);

	        $barCodeOptions = array(
                'height'   => 40,
                'width'    => 0,
                'position' => 0,
                'font'     => 0,
                'margin'   => 25
            );

	        if($NRSEQVENDA != ''){
	        	$barCodeOrder = 'h'.$CDCAIXA.$CDFILIAL.$NRSEQVENDA;
	        }else{
	        	$barCodeOrder = 'h'.$CDFILIAL.$NRVENDAREST.$NRCOMANDA;
	        }
            $comandos->barCode($barCodeOrder, $barCodeOptions);
			$comandos->cutPaper();

			if($IDIMPPRODAPETIC == 'N' || !$IDIMPPRODAPETIC){
				$result = $this->impressaoUtil->requisicaoPonte($dadosImpressora, $comandos);
				$result = $this->imprimeEtiquetaProdutos($produtos, $params, $MODALIDADE, $SENHA, $imprimeMesaPosicao);
			}else{
				$result = $this->imprimeEtiquetaProdutos($produtos, $params, $MODALIDADE, $SENHA, $imprimeMesaPosicao);
			}

	    }else{
	    	$result['error'] = true;
	    	$result['message'] = 'Não foi possivel obter os dados da impressora.';
	    }
		return $result;
	}

	public function imprimeEtiquetaProdutos($produtos, $params, $MODALIDADE, $SENHA, $imprimeMesaPosicao){
		//buscar dados da impressora
		$CDCAIXA =  $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_CDCAIXA_BY_CDSETOR, array('CDSETOR' => $produtos[0]['CDSETOR']));
		$CDCAIXA = $CDCAIXA['CDCAIXA'];
		$dadosImpressora = $this->impressaoUtil->getDadosImpressora($params['CDFILIAL'], $CDCAIXA);

		$dadosImpressora = $dadosImpressora['dadosImpressora'];
		$params['CDCAIXA'] = $CDCAIXA;
		$r_valImprLoja = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_VAL_IMPR_LOJA, $params);
		if (!$r_valImprLoja || empty($r_valImprLoja)) {
			$result['error'] = true;
	    	$result['message'] = 'Não foi possivel obter os dados da impressora.';
			return $result;
		}
		$printerParams = $this->impressaoUtil->buscaParametrosImpressora($r_valImprLoja['IDMODEIMPRES']);
		$printerParams['letterType'] = $printerParams['tipoLetra'];
		$printerParams['italic'] = true;
		$printerParams['bold'] = true;
		$printerParams['expanded'] = true;

		$comandos = new Command();


		foreach ($produtos as $produto) {
			if($produto['IDIMPRPRODPICK'] != 'N'){
				$printQrCodePicklist = $this->instanceManager->getParameter("printQrCodePicklist");
				for ($i = 0; $i < $produto['QTPRODPEFOS']; $i++) {
					$qrCodeTextArray = array();
					if ($printQrCodePicklist) {
						$qrCodeTextArray['CDFILIAL'] = $params['CDFILIAL'];
						$qrCodeTextArray['CDPRODUTO'] = $produto['CDPRODUTO'];
						$qrCodeTextArray['TXPRODPED'] = $produto['observacoesTexto'];
						$qrCodeTextArray['NMPRODUTO'] = $produto['NMPRODUTO'];
						$qrCodeTextArray['CDARVPROD'] = $produto['CDARVPROD'];
						$qrCodeTextArray['NRPEDIDOFOS'] = $params['NRPEDIDOFOS'];
						$qrCodeTextArray['CDBARPRODUTO'] = $produto['CDBARPRODUTO'];
					}

					$useNewLayoutPicklist = $this->instanceManager->getParameter("useNewLayoutPicklist");

					if ($useNewLayoutPicklist) { // imprime no novo layout
						$produto['NMCONSUMIDOR'] = isset($produto['NMCONSUMIDOR']) ? $produto['NMCONSUMIDOR'] : "";

						$textMesa = $MODALIDADE . ' - ' . $SENHA;

						if ($imprimeMesaPosicao) {
							$MESA = isset($produto['NRMESA']) ? $produto['NRMESA'] : "";
							if ($MESA != "") {
								$textMesa = $MODALIDADE . ': ' . $MESA . ' - Pedido: ' . $SENHA;
							} else {
								$textMesa = $MODALIDADE . ' - Pedido: ' . $SENHA;
							}

							$POSICAO = isset($produto['NRLUGARMESAIT']) ? $produto['NRLUGARMESAIT'] : "";
							if ($POSICAO != "") {
								$textMesa .= ' - Posicao: ' . $POSICAO;
							} else {
								$POSICAO = isset($produto['DSLUGARMESAIT']) ? explode(" ", $produto['DSLUGARMESAIT'])[1] : "";
								if ($POSICAO != "") {
									$textMesa .= ' - Posicao: ' . $POSICAO;
								}
							}
						}

						$text = array(
							"mesa" => $textMesa,
							"nomeCliente" => $produto['NMCONSUMIDOR'],
							"pedido" => $produto['NMPRODUTO'],
							"modificacoes" => $produto['observacoesArray'],
							"QRcode" => $printQrCodePicklist ? json_encode($qrCodeTextArray, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES) : null
						);

						$comandos->printESCPOS($text, $printerParams);
						$comandos->cutPaper();
					} else { // imprime no layout antigo
						$text = $MODALIDADE . ' - ' . $SENHA . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
						$text .= chr(27).chr(33).chr(48);
						if(isset($produto['NMCONSUMIDOR'] )){
							$text .= $this->impressaoUtil->alinhaEsquerda($produto['NMCONSUMIDOR'], $printerParams['largura']);
						}

						$text .=  $this->impressaoUtil->alinhaEsquerda($produto['NMPRODUTO'], $printerParams['largura']) . $printerParams['comandoEnter'];
						if($produto['TXPRODPED'] != null){
							foreach($produto['TXPRODPED'] as $key => $obsproduto){
								$APELIDO = empty($obsproduto['DSAPELIDOOBS']) ? $obsproduto['OBSERVACAO'] : $obsproduto['DSAPELIDOOBS'];
								if($APELIDO != ''){
									$text .=  '   '. $obsproduto['IDCONTROLAOBS'] . $APELIDO . $printerParams['comandoEnter'];
								}
							}
						}

						//monta QR Code com informações do produto
						$qrCodeOptions = array(
							'height'   => 40,
							'width'    => 0,
							'position' => 0,
							'font'     => 0,
							'margin'   => 25
						);

						$comandos->text($text, $printerParams);
						if ($printQrCodePicklist)
							$comandos->qrCode(json_encode($qrCodeTextArray, JSON_UNESCAPED_UNICODE), $qrCodeOptions);
						$comandos->cutPaper();
					}
				}
			}
		}

		$result['error'] = false;
        $result = $this->impressaoUtil->requisicaoPonte($dadosImpressora, $comandos);
        return $result;
	}

	private function getParamsProdutos($produtos){
		$produtoBase = $produtos[0];

		$params = array(
			'NRPEDIDOFOS'	=>	$produtoBase['NRPEDIDOFOS'],
			'CDFILIAL'		=>  $produtoBase['CDFILIAL'],
			'NRVENDAREST'	=>  $produtoBase['NRVENDAREST'],
			'NRCOMANDA'		=>  $produtoBase['NRCOMANDA'],
			'NRPRODCOMVEN'  => 	$produtoBase['NRPRODCOMVEN'],
			'NRSEQVENDA'	=>  $produtoBase['NRSEQVENDA'],
			'NRSEQUITVEND'  =>  isset($produtoBase['NRSEQUITVEND']) ? $produtoBase['NRSEQUITVEND'] : '',
		);

		$result = array(
			'CDFILIAL' 		=> $produtoBase['CDFILIAL'],
			'IDORIGEMVENDA' => $produtoBase['IDORIGEMVENDA'],
			'NRPEDIDOFOS'	=> $produtoBase['NRPEDIDOFOS'],
			'NRSEQVENDA'	=> $produtoBase['NRSEQVENDA'],
			'DTHORA' 		=> date('d/m/Y H:i:s'),
			'NRCOMANDA'		=> $produtoBase['NRCOMANDA'],
			'NRVENDAREST'	=> $produtoBase['NRVENDAREST']
		);

		if($produtoBase['CDCAIXA'] == ''){
			$infoComanda = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_CMDINFO, $params);
			$result['CDCAIXA'] = $infoComanda['CDCAIXA'];
			$result['NMCAIXA'] = $infoComanda['NMCAIXA'];
			$result['CDLOJA'] = $infoComanda['CDLOJA'];
			$result['NMLOJA'] = $infoComanda['NMLOJA'];
			$result['CDSENHAPED'] = $infoComanda['CDSENHAPED'];
		}else{
			$infoVenda = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_VNDINFO, $params);
			$result['CDCAIXA'] = $infoVenda['CDCAIXA'];
			$result['NMCAIXA'] = $infoVenda['NMCAIXA'];
			$result['CDLOJA'] = $infoVenda['CDLOJA'];
			$result['NMLOJA'] = $infoVenda['NMLOJA'];
			$result['CDSENHAPED'] = $infoVenda['CDSENHAPED'];
		}
		return $result;
	}

	private function getInformacoesProdutos($produtos){
		foreach ($produtos as $key => $produto) {
			$params = array(
				'CDFILIAL'		=>  $produto['CDFILIAL'],
				'NRPEDIDOFOS'	=>	$produto['NRPEDIDOFOS'],
				'NRITPEDIDOFOS' =>  $produto['NRITPEDIDOFOS'],
				'CDPRODUTO' 	=>  $produto['CDPRODUTO']
			);

			$infoProduto = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_PRODUCT_INFO, $params);
			$produtos[$key]['NMPRODUTO'] = $infoProduto['NMPRODUTO'];
			$produtos[$key]['CDARVPROD'] = $infoProduto['CDARVPROD'];
			$produtos[$key]['CDBARPRODUTO'] = $infoProduto['CDBARPRODUTO'];

			$TXPRODPED = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::SQL_GET_TXPRODPED, $params);
			if (!empty($TXPRODPED['TXPRODPED'])){
				$OBSPROD = explode(';', $TXPRODPED['TXPRODPED']);
			}
			else $OBSPROD = array();
			$produtos[$key]['OBSPROD'] = $OBSPROD;

		}
		return $produtos;
	}

}