<?php

namespace Service;

class Statistics {

	protected $entityManager;
	protected $util;
	protected $query;
	protected $environment;
	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Util\Util $util,
		\Util\Query $query,
		\Lib\Environment $environment,
		$shouldLog
		) {

		$this->entityManager = $entityManager;
		$this->util = $util;
		$this->query = $query;
		$this->connection = $entityManager->getConnection();
		$this->shouldLog = $shouldLog;
		$this->environment = $environment;
	}

	private function getExibEstatics ($paramCDFILIAL) {
		/* verifica no banco se as estatisticas estão habilitadas através do campo IDEXIBESTATIS
		da tabela PARAVEND
		*/
		$query = $this->query;
		$getExibEstatics = $this->connection->fetchAssoc($query::GET_IDEXIBESTATIS, $paramCDFILIAL);
		return $getExibEstatics;
	}

	private function logcheckupdate ($event, $space = false) {
		if (isset($this->shouldLog) && $this->shouldLog === true){
			try {
				if (!isset($this->logTimer[$event])) {
					$this->logTimer[$event] = round(microtime(true) * 1000);
				} else {
					$tempoFinal = round(microtime(true) * 1000);
					$total = $tempoFinal - $this->logTimer[$event];
					//testa se a pasta existe e cria caso não exista
					$folder = substr(__DIR__, 0, 1) . ":\\TEKNISA\\kds\\logs\\";
					if (!file_exists($folder)) {
						mkdir($folder, 0777, true);
					}
					//--grava o log
					$space = $space ? "\n\r\n\r" : "";
					$line = date("d-m-Y H:i:s") . " - Tempo - $event: " . $total . "\r $space";
					$drive = substr(__DIR__, 0, 1);
					$date = date("dmY");
					file_put_contents($drive . ":\\TEKNISA\\kds\\logs\\statisticsLog" . $date . ".txt", $line, FILE_APPEND);
					unset($this->logTimer[$event]);
				}
			} catch (\Exception $e){
				    $this->util->logException($e->getMessage());
			}
		}
    }

	public function playItemStatistics ($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $CDSETOR = NULL) {
		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);

		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);
		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.

		if ($getExibEstatics["IDEXIBESTATIS"] == "S") {
			$params = array(
				'CDFILIAL'      => $CDFILIAL,
				'NRPEDIDOFOS'   => $NRPEDIDOFOS,
				'NRITPEDIDOFOS' => $NRITPEDIDOFOS,
				'CDSETOR'		=> $CDSETOR
			);
			$this->logcheckupdate("playItemStatistics - SQL_GET_ITEM_DATA");
			$maxTries = 10;
			$success = false;
			$error = null;
			do {
				try {
					if ($NRITPEDIDOFOS) {
						// se tiver NIITPEDIDOFOS o pedido não esta agrupado
						$itemsData = $this->connection->fetchAll($query::SQL_GET_ITEM_DATA, $params);
					} else {
						$itemsData = $this->connection->fetchAll($query::SQL_GET_ALL_ITEMS_DATA, $params);
					}
					$success = true;
				} catch (\Exception $e) {
					$success = false;
					$maxTries --;
					$error = $e->getMessage();
					if ($maxTries < 5) {
						sleep(1);
					} else {
						sleep(0.5);
					}
				}
			} while ($maxTries > 0 && !$success);

			if (!$success) {
				$this->util->logException('playItemStatistics - SQL_GET_ITEM_DATA ' . $error);
			}
			$this->logcheckupdate("playItemStatistics - SQL_GET_ITEM_DATA");
			if (!empty($itemsData) && $success) {
				foreach ($itemsData as $itemData) {
					$maxTries = 10;
					$success = false;
					$error = null;
					do {
						try {
							$CDSETOR 	 = $itemData['CDSETOR'];
							$TIMETOSTART = $itemData['TIMETOSTART'];
							$QTPRODPEFOS = $itemData['QTPRODPEFOS'];
							$QTPRODPEFOS = floatval($QTPRODPEFOS);
							$params = array(
								'TIMETOSTART' => $TIMETOSTART,
								'CDFILIAL'    => $CDFILIAL,
								'CDSETOR'     => $CDSETOR,
								'QTPRODPEFOS' => $QTPRODPEFOS <= 0 ? 1 : $QTPRODPEFOS,
							);
							$this->logcheckupdate("playItemStatistics - deleteOldStatistics");
							$this->deleteOldStatistics();
							$this->logcheckupdate("playItemStatistics - deleteOldStatistics");
							$this->logcheckupdate("playItemStatistics - sectorStatistics");
							$sectorStatistics = $this->getSectorStatistics($CDFILIAL, $CDSETOR);
							$this->logcheckupdate("playItemStatistics - sectorStatistics");
							if(empty($sectorStatistics)) {
								$this->logcheckupdate("playItemStatistics - SQL_INSERT_STATISTICS_ON_PLAY");
								$this->connection->executeQuery($query::SQL_INSERT_STATISTICS_ON_PLAY, $params);
								$this->logcheckupdate("playItemStatistics - SQL_INSERT_STATISTICS_ON_PLAY");
							} else {
								$this->logcheckupdate("playItemStatistics - SQL_UPDATE_STATISTICS_PLAY");
								$this->connection->executeQuery($query::SQL_UPDATE_STATISTICS_PLAY, $params);
								$this->logcheckupdate("playItemStatistics - SQL_UPDATE_STATISTICS_PLAY", true);
							}
							$success = true;
						} catch (\Exception $e) {
							$success = false;
							$maxTries --;
							$error = $e->getMessage();
							if ($maxTries < 5) {
								sleep(1);
							} else {
								sleep(0.5);
							}
						}
					} while ($maxTries > 0 && !$success);

					if (!$success) {
						$this->util->logException('playItemStatistics - SQL_INSERT_STATISTICS_ON_PLAY || SQL_UPDATE_STATISTICS_PLAY ' . $error);
					}
				}
			}
		}
	}
	/** grava estatistica da produção */
	public function concludeItemStatistics($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);

		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);
		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.
		if($getExibEstatics["IDEXIBESTATIS"] == "S") {
			$params = array(
				'CDFILIAL'      => $CDFILIAL,
				'NRPEDIDOFOS'   => $NRPEDIDOFOS,
				'NRITPEDIDOFOS' => $NRITPEDIDOFOS
			);
			$this->logcheckupdate("concludeItemStatistics - SQL_GET_ITEM_DATA");
			$maxTries = 10;
			$success = false;
			$error = null;
			do {
				try {
					$itemData = $this->connection->fetchAssoc($query::SQL_GET_ITEM_DATA, $params);
					$success = true;
				} catch (\Exception $e) {
					$success = false;
					$maxTries --;
					$error = $e->getMessage();
					if ($maxTries < 5) {
						sleep(1);
					} else {
						sleep(0.5);
					}
				}
			} while ($maxTries > 0 && !$success);

			if (!$success) {
				$this->util->logException('concludeItemStatistics - SQL_GET_ITEM_DATA ' . $error);
			}
			$this->logcheckupdate("concludeItemStatistics - SQL_GET_ITEM_DATA");
			if (!empty($itemData) && $success) {
				$CDSETOR = $itemData['CDSETOR'];
				$itemData['TIMETOPRODUCE'] = $this->calculateTimeToProduce($CDFILIAL, $CDSETOR, $itemData);
				$NRTEMPOPRODIT = $itemData['NRTEMPOPRODIT'];
				$QTPRODPEFOS = $itemData['QTPRODPEFOS'];
				$QTPRODPEFOS = floatval($QTPRODPEFOS);

				// Salvar na tabela temporária
				$this->logcheckupdate("concludeItemStatistics - includeSetorExt");
				$this->includeSetorExt($CDFILIAL, $itemData['CDSETOR'], $NRPEDIDOFOS, $itemData['TIMETOPRODUCE'], 'concludeItemStatistics '.json_encode($itemData));
				$this->logcheckupdate("concludeItemStatistics - includeSetorExt");

				$this->logcheckupdate("concludeItemStatistics - calculateAndConclude");
				$this->calculateAndConclude($CDFILIAL, $CDSETOR, $itemData['TIMETOPRODUCE'], $NRTEMPOPRODIT, $QTPRODPEFOS);
				$this->logcheckupdate("concludeItemStatistics - calculateAndConclude");
				
				$this->logcheckupdate("concludeItemStatistics - registrySectorConclusionQuants");
				$this->registrySectorConclusionQuants($CDFILIAL, $itemData['CDSETOR']);
				$this->logcheckupdate("concludeItemStatistics - registrySectorConclusionQuants", true);
			}
		}
	}

	private function calculateAndConclude($CDFILIAL, $CDSETOR, $TIMETOPRODUCE, $NRTEMPOPRODIT, $QTPRODPEFOS) {
		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);

		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);

		if ($getExibEstatics["IDEXIBESTATIS"] == "S") {
			$maxTries = 10;
			$success = false;
			$error = null;
			do {
				$QTPRODPEFOS = is_null($QTPRODPEFOS) || $QTPRODPEFOS <= 0 ? 1 : $QTPRODPEFOS;
				if ($TIMETOPRODUCE > $NRTEMPOPRODIT) {
					$DELAYTIME = $TIMETOPRODUCE - $NRTEMPOPRODIT;
					$QTATRASADO = $QTPRODPEFOS;
				} else {
					$DELAYTIME = 0;
					$QTATRASADO = 0;
				}

				$params = array(
					'TIMETOPRODUCE' => $TIMETOPRODUCE,
					'DELAYTIME'     => $DELAYTIME,
					'QTATRASADO'    => $QTATRASADO,
					'CDFILIAL'      => $CDFILIAL,
					'CDSETOR'       => $CDSETOR,
					'QTPRODPEFOS'   => $QTPRODPEFOS
				);

				// Corrige um problema causado por pedidos expedidos antes do tempo de produção iniciar 
                // (pedidos "cinza" no KDS com tempo decrescente), que gravava valores negativos no banco
                if ($params['TIMETOPRODUCE'] < 0){
                    $params['TIMETOPRODUCE'] = 0;
                }

				//Deleta as estatisticas antigas da tabela ITPEDIDOEST
				$this->logcheckupdate("calculateAndConclude - deleteOldStatistics");
				$this->deleteOldStatistics();
				$this->logcheckupdate("calculateAndConclude - deleteOldStatistics");
				$this->logcheckupdate("calculateAndConclude - getSectorStatistics");

				$sectorStatistics = $this->getSectorStatistics($CDFILIAL, $CDSETOR);
				$this->logcheckupdate("calculateAndConclude - getSectorStatistics");
				if(empty($sectorStatistics)) {
					$this->logcheckupdate("calculateAndConclude - SQL_INSERT_STATISTICS_ON_CONCLUSION");
					$this->connection->executeQuery($query::SQL_INSERT_STATISTICS_ON_CONCLUSION, $params);
					$this->logcheckupdate("calculateAndConclude - SQL_INSERT_STATISTICS_ON_CONCLUSION");
					$this->logcheckupdate("calculateAndConclude - SQL_UPDATE_STATISTICS_CONCLUSION");
					$this->connection->executeQuery($query::SQL_UPDATE_STATISTICS_CONCLUSION, $params);
					$this->logcheckupdate("calculateAndConclude - SQL_UPDATE_STATISTICS_CONCLUSION" , true);
				} else  {								
					$this->logcheckupdate("calculateAndConclude - SQL_UPDATE_STATISTICS_CONCLUSION");
					$this->connection->executeQuery($query::SQL_UPDATE_STATISTICS_CONCLUSION, $params);
					$this->logcheckupdate("calculateAndConclude - SQL_UPDATE_STATISTICS_CONCLUSION", true);
				}
				$success = true;
			} while ($maxTries > 0 && !$success);
			if (!$success) {
				$this->util->logException('calculateAndConclude ' . $error);
			}
		}
	}

	/** grava estatistica da expedição */
	public function expediteOrderStatistics($CDFILIAL, $CDSETOR, $itemsArray) {
		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);
		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);

		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.
		if ($getExibEstatics["IDEXIBESTATIS"] == "S") {
			try {
				foreach ($itemsArray as $currentItem) {
					$params = array(
						'CDFILIAL'      => $currentItem['CDFILIAL'],
						'NRPEDIDOFOS'   => $currentItem['NRPEDIDOFOS'],
						'NRITPEDIDOFOS' => $currentItem['NRITPEDIDOFOS']
					);

					$this->logcheckupdate("expediteOrderStatistics - SQL_GET_ITEM_DATA");
					$itemData = $this->connection->fetchAssoc($query::SQL_GET_ITEM_DATA, $params);
					$this->logcheckupdate("expediteOrderStatistics - SQL_GET_ITEM_DATA");

					if (!empty($itemData)) {
						$itemData['TIMETOPRODUCE'] = $this->calculateTimeToProduce($CDFILIAL, $CDSETOR, $itemData);
						$NRTEMPOPRODIT = $itemData['NRTEMPOPRODIT'];
						$QTPRODPEFOS = $itemData['QTPRODPEFOS'];
						$QTPRODPEFOS = floatval($QTPRODPEFOS);

						$this->logcheckupdate("expediteOrderStatistics - includeSetorExt");
						$this->includeSetorExt($CDFILIAL, $itemData['CDSETOR'], $currentItem['NRPEDIDOFOS'], $itemData['TIMETOPRODUCE'], 'expediteOrderStatistics '.json_encode($itemData));
						$this->logcheckupdate("expediteOrderStatistics - includeSetorExt");
						
						$this->logcheckupdate("expediteOrderStatistics - calculateAndConclude");
						$this->calculateAndConclude($CDFILIAL, $CDSETOR, $itemData['TIMETOPRODUCE'], $NRTEMPOPRODIT, $QTPRODPEFOS);
						$this->logcheckupdate("expediteOrderStatistics - calculateAndConclude");
						
						$this->logcheckupdate("expediteOrderStatistics - registrySectorConclusionQuants");
						$this->registrySectorConclusionQuants($CDFILIAL, $itemData['CDSETOR'], $currentItem['NRPEDIDOFOS']);
						$this->logcheckupdate("expediteOrderStatistics - registrySectorConclusionQuants", true);
					}
				}
			}  catch(\Exception $e){
				$this->util->logException($e->getMessage());
			}
		}
	}

    /**
     * Calcula e retorna o $TIMETOPRODUCE que será utilizado para os cálculos de estatísticas,
     * levando em consideração a parametrização do campo IDFRNONNTF (congelar o tempo na expedição após notificação).
     */
    private function calculateTimeToProduce($CDFILIAL, $CDSETOR, $itemData) {

        $TIMETOPRODUCE = $itemData['TIMETOPRODUCE'];

        $freeze_time_params = array(
            ':CDSETOR' => $CDSETOR,
            ':CDFILIAL' => $CDFILIAL
        );

        $freezeTime = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_FREEZE_TIME_PARAM, $freeze_time_params);

        if ($freezeTime['IDFRNONNTF'] === 'N'){

            $horaFimProducao = strtotime($itemData['DTHRFINPRODUCAO']);
            $horaPriExped = strtotime($itemData['DTHRPRIEXPED']);

            if ($horaPriExped !== false){
                $diferencaEmSegundos = $horaFimProducao - $horaPriExped;
                $TIMETOPRODUCE += $diferencaEmSegundos;
            } 
        }
        return $TIMETOPRODUCE;
    }
    
	public function deleteOldStatistics(){

		/*Função estava apresentando problema quando os restaurantes funcionavam de 00h às 03h, não trazendo 
		as estatísticas de forma correta. Adicionada verificação para consultar a hora do sistema. 
		Caso a hora seja menor do que 3, o delete vai acontecer com estatísticas de dois dias. 
		Caso seja maior do que 3, o delete vai acontecer com todas as estatísticas do dia anterior */

		$query = $this->query;
		$hour = getdate()['hours'];
		if ($hour < 3){
			$this->connection->executeQuery($query::DELETE_TWO_DAYS_STATISTICS);
		} else {
			$this->connection->executeQuery($query::DELETE_OLD_STATISTICS);
		} 
	}

	public function getSectorStatistics($CDFILIAL, $CDSETOR){
		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);
		// alterado para o metodo getExibEstatics 
		//$getExibEstatics = $this->connection->fetchAssoc($query::GET_IDEXIBESTATIS, $paramCDFILIAL);

		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);

		if ($getExibEstatics["IDEXIBESTATIS"] == "S") {
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDSETOR'  => $CDSETOR
			);
		
			//Verifica se as estatisticas do setor estão na variavel de sessão
			$sectorStatisticsExists = $this->environment->getSessionValue("sectorStatisticsExists[$CDSETOR]");
			if(!$sectorStatisticsExists){
				//Essa é a query que não retorna nada na primeira expediçao
				$sectorData = $this->connection->fetchAssoc($query::SQL_GET_STATISTICS_EXISTS, $params);
				if($sectorData){
					/*se houver dados em $sectorData, adiciona na variavel de sessão para que não seja necessario varias 
					consultas ao banco */
					$this->environment->setSessionValue("sectorStatisticsExists[$CDSETOR]", $sectorData);
					$sectorStatisticsExists = $sectorData;
				}				
			}
			return $sectorStatisticsExists;
		}
	}
	
	public function getSectorStatisticsFromTemp($CDFILIAL, $CDSETOR){
		$day = getdate()['mday'];
		$hour = getdate()['hours'];
		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);

		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);
		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.

		if ($getExibEstatics["IDEXIBESTATIS"] == "S") {
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDSETOR'  => $CDSETOR
			);

			$sectorStatisticsExistsTemp = $this->environment->getSessionValue("sectorStatisticsExistsTemp[$CDSETOR]");

			 if (!$sectorStatisticsExistsTemp || (isset($sectorStatisticsExistsTemp['IDDIA']) != $day)){
				//Essa é a query que não retorna nada na primeira expediçao
				$sectorData = $this->connection->fetchAssoc($query::GET_TEMP_STATISTICS_BY_CDSETOR, $params);
				if ($sectorData){
					$sectorData['IDDIA'] = $day;
					$sectorData['IDHORA'] = $hour;
					// se houver dados em $sectorData, coloque na variavel de sessão para que não seja necesasrio varias 
					// consultas ao banco
					$this->environment->setSessionValue("sectorStatisticsExistsTemp[$CDSETOR]", $sectorData);
					$sectorStatisticsExistsTemp = $sectorData;
				}				
			}
			return $sectorStatisticsExistsTemp;
		}
	}
	
	public function registrySectorConclusionQuants($CDFILIAL, $CDSETOR){

		//temporariamente desabilitado, calculo errado causando travamento do KDS por não processar evento da kdsoperacaotemp
		
		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);

		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);
		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.

		if ($getExibEstatics["IDEXIBESTATIS"] == "S") {
			$existentStatistics = $this->getSectorStatistics($CDFILIAL, $CDSETOR);

			$maxTries = 10;
			$success = false;
			$error = null;

			do {
				try {
					if(empty($existentStatistics)){
						// essa linha deu erro na primeira expedicao
						$this->connection->executeQuery($query::SQL_INSERT_STATISTICS, Array('CDFILIAL' => $CDFILIAL, 'CDSETOR' => $CDSETOR));
					}
					/*
						CHAMAR ROTINA DE ESTATISTICAS BASEADA NA TABELA TEMPORARIA 
						MDTOTAL.CDSETOR, MEDIAGERAL, MEDIAGERALHRANT, MEDIAGERALHRATUAL, MEDIATOTAL, MEDIAHORAANTERIOR, QTTOTAL, QTHORAATUAL, QTHORAANTERIOR, QTGERAL, QTGERALHRANT, QTGERALHRATUAL, MEDIAHORAATUAL
					*/
					$existentStatisticsTemp = $this->getSectorStatisticsFromTemp($CDFILIAL, $CDSETOR);
					$params = array(
						'CDFILIAL'		=> $CDFILIAL,
						'CDSETOR'		=> $CDSETOR,
						'IDDIA'			=> getdate()['mday'],
						'IDHORA'		=> getdate()['hours'],
						'QTDIA'			=> $existentStatisticsTemp['QTTOTAL'],
						'QTHORA'		=> $existentStatisticsTemp['QTHORAATUAL'],
						'QTHORAANT' 	=> $existentStatisticsTemp['QTHORAANTERIOR'],
						'MEDIAHORAANT' 	=> (float)$existentStatisticsTemp['MEDIAHORAANTERIOR'],
						'MEDIADIA'	 	=> (float)$existentStatisticsTemp['MEDIATOTAL'],
						'MEDIAHORA' 	=> (float)$existentStatisticsTemp['MEDIAHORAATUAL'],
					);
					$this->connection->executeQuery($query::SQL_UPDATE_NEW_STATISTICS, $params);
					$success = true;
				} catch (\Exception $e){
					$maxTries --;
					$error = $e->getMessage();
					if ($maxTries < 5) {
						sleep(1);
					} else {
						sleep(0.5);
					}
				}
			} while ($maxTries > 0 && !$success);

			if (!$success) {
				$this->util->logException('registrySectorConclusionQuants: ' . $error);
			}
		}
	}

	public function activateOverallStatistics($CDFILIAL, $CDSETOR){
		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);

		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);
		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.

		if ($getExibEstatics["IDEXIBESTATIS"] == "S") {
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDSETOR'  => $CDSETOR
			);
			
			$this->connection->executeQuery($query::UPDATE_SHOW_OVERALL_STATISTICS, $params);
		}
	}

	private function includeSetorExt($CDFILIAL, $CDSETOR, $NRPEDIDOFOS, $NRTEMPOTOTAL, $METODO = '') {
		//$CDFILIAL, $expedition['CDSETOR'], $expedition['NRPEDIDOFOS'], $expedition['NRTEMPOTOTAL'], $expedition['DTHRFIMPRODUCAO']

		$query = $this->query;
		$paramCDFILIAL = array (
			':CDFILIAL' => $CDFILIAL
		);

		$getExibEstatics = $this->getExibEstatics($paramCDFILIAL);
		// $getExibEstatics vai verificar na query se está habilitado a exibição das estatíscas.
		if ($getExibEstatics["IDEXIBESTATIS"] == "S") {
			$query = $this->query;
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDSETOR' => $CDSETOR,
				'NRPEDIDOFOS'  => $NRPEDIDOFOS,
				'NRTEMPOTOTAL' => $NRTEMPOTOTAL  
			);
				
			// Corrige um problema causado por pedidos expedidos antes do tempo de produção iniciar 
            // (pedidos "cinza" no KDS com tempo decrescente), que gravava valores negativos no banco
            if ($params['NRTEMPOTOTAL'] < 0){
                $params['NRTEMPOTOTAL'] = 0;
            }

			$exists = null;

			$maxTries = 10;
			$success = false;
			$error = null;
			do {
				try {
					$exists = $this->connection->fetchAssoc($query::GET_ITEM_FROM_KDS_TEMP_SETOREST, $params);
					if (!$exists) {
						$this->connection->executeQuery($query::INSERT_INTO_KDS_TEMP_SETOREST, $params);
						$this->localStatistics($params);
					}
					$success = true;
				} catch(\Exception $e){
					$success = false;
					$maxTries --;
					$error = $e->getMessage();
					if ($maxTries < 5) {
						sleep(1);
					} else {
						sleep(0.5);
					}
				}
			} while ($maxTries > 0 && !$success);

			if (!$success) {
				$this->util->logException('includeSetorExt ' . $error);
			}
			// Este aqui é chamado
			// Insere na kds_temp_setorest
		}
	}

	private function localStatistics($params) {
	 	$DAY = getdate()['mday'];
		$HOUR = getdate()['hours'];
		$NRTEMPOTOTAL = $params['NRTEMPOTOTAL'];
		$CDSETOR = $params['CDSETOR'];
		$CDFILIAL = $params['CDFILIAL'];

		$sectorStatisticsLocal = $this->environment->getSessionValue("sectorStatisticsExistsTemp[$CDSETOR]");
		if($sectorStatisticsLocal && (isset($sectorStatisticsLocal['IDDIA']) == $DAY)){
			//MEDIA GERAL NÃO DEPENDE DA HORA
			$sectorStatisticsLocal['QTTOTAL']++; //Quantidade total de pedidos do setor
			$sectorStatisticsLocal['NRTEMPOTOTAL'] = (int)$sectorStatisticsLocal['NRTEMPOTOTAL'] + $NRTEMPOTOTAL;
			$sectorStatisticsLocal['MEDIAGERAL'] = (float)($sectorStatisticsLocal['NRTEMPOTOTAL'] / $sectorStatisticsLocal['QTTOTAL']);
			//MEDIA TOTAL (MESMO CALCULO DA MEDIA GERAL)
			$sectorStatisticsLocal['MEDIATOTAL'] = (float)$sectorStatisticsLocal['MEDIAGERAL'];

			if ($sectorStatisticsLocal['IDHORA'] == $HOUR) {
				//MEDIA GERAL HORA ANTERIOR
				$sectorStatisticsLocal['QTGERALHRANT'];
				$sectorStatisticsLocal['TEMPOTOTALGERALHRANT'] =(int) $sectorStatisticsLocal['TEMPOTOTALGERALHRANT'];
				$sectorStatisticsLocal['MEDIAGERALHRANT'] = (float)$sectorStatisticsLocal['MEDIAGERALHRANT'];
				// MEDIAGERAL HORA ATUAL
				$sectorStatisticsLocal['TEMPOTOTALGERALHRAAT'] = (int)$sectorStatisticsLocal['TEMPOTOTALGERALHRAAT'] + $NRTEMPOTOTAL;
				$sectorStatisticsLocal['QTGERALHRATUAL'] ++;
				$sectorStatisticsLocal['MEDIAGERALHRATUAL'] = (float)($sectorStatisticsLocal['TEMPOTOTALGERALHRAAT'] / $sectorStatisticsLocal['QTGERALHRATUAL']);
				//MEDIA HORA ATUAL
				$sectorStatisticsLocal['QTHORAATUAL']++;
				$sectorStatisticsLocal['TEMPOTOTALHORAAT'] = $sectorStatisticsLocal['TEMPOTOTALHORAAT'] + $NRTEMPOTOTAL;			
				$sectorStatisticsLocal['MEDIAHORAATUAL'] = (float)($sectorStatisticsLocal['TEMPOTOTALHORAAT'] / $sectorStatisticsLocal['QTHORAATUAL']);
				//MEDIA HORA ANTERIOR
				$sectorStatisticsLocal['QTHORAANTERIOR'] = (int)$sectorStatisticsLocal['QTHORAANTERIOR'];
				$sectorStatisticsLocal['TEMPOTOTALHORAANTERIOR'] = (int)$sectorStatisticsLocal['TEMPOTOTALHORAANTERIOR'];
				$sectorStatisticsLocal['MEDIAHORAANTERIOR'] = (float)$sectorStatisticsLocal['MEDIAHORAANTERIOR'];
				
				$sectorStatisticsLocal['IDHORA'] = $HOUR;
				$sectorStatisticsLocal['IDDIA'] = $DAY;
				$sectorStatisticsLocal['CDSETOR'] = $CDSETOR;

			} else {

				//MEDIA GERAL HORA ANTERIOR
				$sectorStatisticsLocal['QTGERALHRANT'] = (int)$sectorStatisticsLocal['QTGERALHRATUAL'];
				$sectorStatisticsLocal['TEMPOTOTALGERALHRANT'] = (int)$sectorStatisticsLocal['TEMPOTOTALGERALHRAAT'];
				$sectorStatisticsLocal['MEDIAGERALHRANT'] = (float)$sectorStatisticsLocal['MEDIAGERALHRATUAL'];
				// MEDIAGERAL HORA ATUAL
				$sectorStatisticsLocal['TEMPOTOTALGERALHRAAT'] = $NRTEMPOTOTAL;
				$sectorStatisticsLocal['QTGERALHRATUAL'] = 1;
				$sectorStatisticsLocal['MEDIAGERALHRATUAL'] = (float)($sectorStatisticsLocal['TEMPOTOTALGERALHRAAT'] / $sectorStatisticsLocal['QTGERALHRATUAL']);
				//MEDIA HORA ANTERIOR
				$sectorStatisticsLocal['QTHORAANTERIOR'] = (int)$sectorStatisticsLocal['QTHORAATUAL'];
				$sectorStatisticsLocal['TEMPOTOTALHORAANTERIOR'] = (int)$sectorStatisticsLocal['TEMPOTOTALHORAAT'];
				$sectorStatisticsLocal['MEDIAHORAANTERIOR'] = (float)$sectorStatisticsLocal['MEDIAHORAATUAL'];
				//MEDIA HORA ATUAL
				$sectorStatisticsLocal['QTHORAATUAL'] = 1;
				$sectorStatisticsLocal['TEMPOTOTALHORAAT'] = $NRTEMPOTOTAL;				
				$sectorStatisticsLocal['MEDIAHORAATUAL'] = (float)($sectorStatisticsLocal['TEMPOTOTALHORAAT'] / $sectorStatisticsLocal['QTHORAATUAL']);
			
				$sectorStatisticsLocal['IDHORA'] = $HOUR;
				$sectorStatisticsLocal['IDDIA'] = $DAY;
				$sectorStatisticsLocal['CDSETOR'] = $CDSETOR;
			}
			$this->environment->setSessionValue("sectorStatisticsExistsTemp[$CDSETOR]", $sectorStatisticsLocal);
		} else {
			// limpa as variaveis de sessão e recalcula as estatisticas
			$this->environment->clearSession();
			$this->registrySectorConclusionQuants($CDFILIAL, $CDSETOR);
		}
	}
}