function DisplayController(ScreenService, environment, AuthController, NavigatorService,
	Utilities, templateManager, SocketServiceKDS, ConcludeQueue, $timeout, $interval, SoundService,
	eventAggregator, ShortcutKeyService, SummaryItemsService, ScreenMessagesService, OrderActionService, RestEngine, ScannerDetection) {

	"use strict";

	var self = this;
	var KDS_SCREEN_PRODUCTION = 'production';
	var KDS_SCREEN_EXPEDITION = 'expedition';
	var intervalUpdateOrderTimers;
	var localEnvironment = {};
	var KDSWidget;

	this.kdsOnEnter = function (widget, type) {
		environment.setKDSWidgetOnEnvironment(widget);
		localEnvironment = environment.getLocalEnvironment();

        // Passa o parâmetro IDHABEXPFLUID para dentro da Widget, para acessá-lo facilmente de dentro do HTML
        widget.IDHABEXPFLUID = localEnvironment.IDHABEXPFLUID;
		// Passa o parâmetro IDHABPRIORIDADE para dentro da Widget, para acessá-lo facilmente de dentro do HTML
        widget.IDHABPRIORIDADE = localEnvironment.IDHABPRIORIDADE;

		self.beginToWork(widget, type);
		let label = "Liberação por Código de Barras";

		//@TODO directive
		widget.getSummaryItems = function () {
			return SummaryItemsService.getSummaryItems();
		};
		widget.orderSummaryItems = function (order) {
			return SummaryItemsService.sortOrders(order);
		};
		if (localEnvironment.IDTIPOSETOR == "E") {
			this.getButtonPosition(label).isVisible = true;
		} else {
			this.getButtonPosition(label).isVisible = false;
		}

	};

	this.getButtonPosition = function (label) {
		return templateManager.project.getMenu("kdsOptions").menus.find(button => button.label == label);
	};

	this.changeStateScanCode = function (widget) {
		localEnvironment = environment.getLocalEnvironment();
		let IDHABSCANCODE = localEnvironment.IDHABSCANCODE == 'S' ? 'N' : 'S';
		widget.IDHABSCANCODE = IDHABSCANCODE;
		localEnvironment.IDHABSCANCODE = IDHABSCANCODE;
		let CDSETOR = localEnvironment.CDSETOR;
		let scanCodeRoute = '/StateScanCode/update';

		var params = {
			requestType: "Row",
			row: {
				"IDHABSCANCODE": IDHABSCANCODE,
				"CDSETOR": CDSETOR
			}
		};

		RestEngine.post(scanCodeRoute, params).then(function (response) {
		}).catch(err => {
			return err;
		})
			.finally(() => {
				window.customKDSReload();
			});
	};

	this.showOverallStatistics = function (widget) {
		localEnvironment = environment.getLocalEnvironment();
		let CDSETOR = localEnvironment.CDSETOR;
		let CDFILIAL = localEnvironment.CDFILIAL;
		let statisticsRoute = "/ShowOverallStatistics/update";

		var params = {
			requestType: "Row",
			row: {
				"CDSETOR": CDSETOR,
				"CDFILIAL": CDFILIAL
			}
		};

		RestEngine.post(statisticsRoute, params).then(function (response) {
			console.log(response.dataset);
		}).catch(err => {
			return err;
		})
			.finally(() => {
				window.customKDSReload();
			});
	};

	this.beginToWork = function (widget, type) {
		widget.IDTIPOSETOR = localEnvironment.IDTIPOSETOR;
		widget.IDHABSCANCODE = localEnvironment.IDHABSCANCODE;
		widget.IDHABLIBDIRETOEXP = localEnvironment.IDHABLIBDIRETOEXP;
		widget.IDHABESTGERAL = localEnvironment.IDHABESTGERAL;
		widget.productionProgressBar = localEnvironment.IDHABTIMERPROD === 'S' && widget.IDTIPOSETOR === 'P';
		widget.container.label = localEnvironment.labelContainer;
		widget.hasSummary = localEnvironment.hasSummary;
		KDSWidget = widget;
		SummaryItemsService.updateSummaryItems();
		ScannerDetection.startScanning();

		// recupera o parametro useDarkMode do localStorage e salva no widget
		widget.useDarkMode = localStorage.getItem('useDarkMode') === 'true';
		handleDarkMode(widget);

		// setTimeout(function(){SocketServiceKDS.getData(localEnvironment.CDFILIAL, localEnvironment.CDSETOR);}, 3333/4);
		if (type == KDS_SCREEN_PRODUCTION) {
			this.handleNotInitializedProducts(widget);
		}

	};

	function handleDarkMode(widget) {
		if (widget.useDarkMode) {
			document.querySelectorAll(".zh-application-content")[0].classList.add("dark-mode");
		}
	}

	function findOrderByOrderKey(orderKey) {
		if (KDSWidget && KDSWidget.dataSource && KDSWidget.dataSource.data) {
			return KDSWidget.dataSource.data.filter(function (currentOrder) {
				return currentOrder.orderKey == orderKey;
			}).shift();
		}
	}

	this.onItemDelete = function (itemsToDelete) {
		if (KDSWidget) {
			Object.keys(itemsToDelete).forEach(function (orderKey) {
				if (localEnvironment.IDTIPOSETOR == 'P') {
                    if (localEnvironment.IDHABAGRUPPROD == "S"){
                        if(KDSWidget && KDSWidget.dataSource && KDSWidget.dataSource.data){
							KDSWidget.dataSource.data.forEach(function(order){
								order.items.forEach(function(orderItem, index, items){
									if(orderItem.NRPEDIDOFOS == itemsToDelete[orderKey].NRPEDIDOFOS &&
										orderItem.NRITPEDIDOFOS == itemsToDelete[orderKey].NRITPEDIDOFOS)
									{
										if(orderItem.NRITPEDIDOFOS == itemsToDelete[orderKey].NRITPEDIDOFOS){
											var now = new moment();
											if(order.LASTUPDATE == now.format()){
												now.add(1, 'second');
											}
											order.LASTUPDATE = now.format();
											items.splice(index, 1);
											// KDSWidget.updateKDSTemplate();
											order.updateAvailableAction();
											order.getOrderColor();
										}
									}
								});
							});
						}
                    } else {
                        var toRemoveOrder = findOrderByOrderKey(orderKey);
                        var toRemoveOrderIndex = KDSWidget.dataSource.data.indexOf(toRemoveOrder);
                        if (~toRemoveOrderIndex) {
                            toRemoveOrder.clearBeeps();
                            //SoundService.clearOrderBeeps(toRemoveOrder);
                            if (toRemoveOrder.timerInterval) {
                                $interval.cancel(toRemoveOrder.timerInterval);
                                delete toRemoveOrder.timerInterval;
                            }
                            NavigatorService.setOrderAsSelected(toRemoveOrderIndex, 'move');
                            KDSWidget.dataSource.data.splice(toRemoveOrderIndex, 1);
                            SummaryItemsService.updateSummaryItems();
                        }
                    }
				} else {
					if (KDSWidget && KDSWidget.dataSource && KDSWidget.dataSource.data) {
						KDSWidget.dataSource.data.forEach(function (order, orderIndex, orders) {
							if (order.NRPEDIDOFOS == itemsToDelete[orderKey].NRPEDIDOFOS) {
								order.items.forEach(function (item, index, items) {
									if (item.groupedItens && item.groupedItens.length > 0) {
										let gItemsToRemove = [];
										item.groupedItens.forEach((gItem) => {
											if (gItem.NRITPEDIDOFOS == itemsToDelete[orderKey].NRITPEDIDOFOS) {
												gItemsToRemove.push(gItem);
											}
										});
										if (gItemsToRemove) {
											gItemsToRemove.forEach((e) => {
												let idx = item.groupedItens.indexOf(e);
												item.groupedItens.splice(idx, 1);
											});
											let items = item.groupedItens;
											order.items.splice(index, 1);
											order.items = items;
										}
									} 
									if (item.NRITPEDIDOFOS == itemsToDelete[orderKey].NRITPEDIDOFOS && !item.groupedItens) {
										items.splice(index, 1);
									}
								});
								let now = new moment();
								if (order.LASTUPDATE == now.format()) {
									now.add(1, 'second');
								}
								// KDSWidget.updateKDSTemplate();
								order.updateAvailableAction();
								order.getOrderColor();
								order.LASTUPDATE = now.format();
								order.items = KDSWidget.checkAndGroupItems(order.items);
							}
						});
					}

				}
			});
			KDSWidget.updateKDSTemplate();
		}
	};

	function findOrderInScreen(orderToCompare) {
		var order = KDSWidget.dataSource.data.filter(function (currentOrder) {
			return checkOrderExistence(currentOrder, orderToCompare.orderKey);
		}).shift();
		return KDSWidget.dataSource.data.indexOf(order);
	}

	//isso serve para no caso do primeiro item de um pedido for trocado, nao duplicar.
	function checkOrderExistence(order, orderKey) {
		if (order.items) {
			return order.items.some(function (item) {
				return item.orderKey == orderKey;
			});
		}
	}

	function checkOrderToAdd(order, senhaped, nrmesa) {
		if(order.items){
			return order.items.some(function(item) {
				return (item.CDSENHAPED == senhaped && item.NRMESA == nrmesa);
			});
		}
	}

	function replaceZoomItem(order) {
		if (KDSWidget.zoomItem) {
			if (checkOrderExistence(KDSWidget.zoomItem, order.orderKey)) {
				KDSWidget.zoomItem = Util.extend(KDSWidget.zoomItem, order, true);
			}
		}
	}
	
	function findOrderGroup(orderToCompare){
		var order = KDSWidget.dataSource.data.filter(function(currentOrder){
			return checkOrderToAdd(currentOrder, orderToCompare.CDSENHAPED, orderToCompare.NRMESA);
		}).shift();
		return KDSWidget.dataSource.data.indexOf(order);
	}

	this.onItemProducaoChange = function (requests) {
		if (KDSWidget) {
			var oldLength = KDSWidget.dataSource.data.length;
			Object.keys(requests).forEach(function (key) {
				if (!ConcludeQueue.checkItemExistence(key)) {
					var indexToReplace = findOrderInScreen(requests[key]);
                    var indexToAgroup = null;

                    if (localEnvironment.IDHABAGRUPPROD == "S") {
                        indexToAgroup = findOrderGroup(requests[key]);
                    }

					if (indexToReplace) {
						replaceZoomItem(requests[key]);
					}
					//isso olha se já esta na hora de mostrar o item que mais demora desse pedido.
					//se não estiver e o pedido não estiver liberado ele vai pro waiting items.

					if ((requests[key].IDLIBERADO == 'N' && localEnvironment.IDHABPREENVIO && requests[key].waiting)) {
						requests[key].timer = 'wait';
						if (moment(requests[key].DTHREXIBICAOPROD).subtract(requests[key].NRMAIORTEMPO, 'seconds').isBefore(requests[key].NOW)) {
							requests[key].IDLIBERADO = 'S';
						}
					}
					if (requests[key].IDLIBERADO == 'S' || requests[key].waiting) {
						if (~indexToReplace && indexToReplace != null) {
							requests[key].isNewOrder = false;

                            if (requests[key].IDSITITPEDFOS == "C" && localEnvironment.IDHABAGRUPPROD == "S") {
                                //Para agrupar um item cancelado de acordo com a resposta do websocket
                                KDSWidget.dataSource.data[indexToReplace].items.forEach(function(item, index, array) {
									// Se o pedido tiver itens cancelados procura o item cancelado e marca como cancelado no front
									var itemToReplace = requests[key].items.find(function(newItem) {
										return newItem.NRPEDIDOFOS == item.NRPEDIDOFOS && newItem.NRITPEDIDOFOS == item.NRITPEDIDOFOS;
									});
									if (itemToReplace.IDSITITPEDFOS == "C") {
										item.IDSITITPEDFOS = "C";
									}
                                    if (requests[key].NRPEDIDOFOS == item.NRPEDIDOFOS && requests[key].NRITPEDIDOFOS == item.NRITPEDIDOFOS) {
                                        item.IDSITITPEDFOS = "C";
                                    }
                                });
								KDSWidget.dataSource.data[indexToReplace].updateAvailableAction();
								KDSWidget.dataSource.data[indexToReplace].getOrderColor();
                            } else if (localEnvironment.IDHABAGRUPPROD == "S" && ~indexToAgroup) {
                                //Para agrupar um item que estava em espera para iniciar de acordo com a resposta do websocket
                                KDSWidget.dataSource.data[indexToReplace].items.forEach(function(item, index, array) {
                                    if (requests[key].NRPEDIDOFOS == item.NRPEDIDOFOS) {
                                        requests[key].items.forEach(function(itemR, indexR, arrayR) {
                                            if (item.NRITPEDIDOFOS == itemR.NRITPEDIDOFOS) {
                                                if(!Array.isArray(itemR.TXPRODPED)){
                                                    if (itemR.TXPRODPED != ''){
                                                        itemR.TXPRODPED = JSON.parse(itemR.TXPRODPED);
                                                    }
                                                }
                                                KDSWidget.dataSource.data[indexToReplace].items[index] = Util.extend(KDSWidget.dataSource.data[indexToReplace].items[index], itemR, true);
                                            }
                                        });
                                    }
                                });
                                KDSWidget.dataSource.data[indexToReplace].updateAvailableAction();
                                KDSWidget.dataSource.data[indexToReplace].getOrderColor();
                                KDSWidget.dataSource.data[indexToAgroup].LASTUPDATE = new moment().format();
                                // KDSWidget.updateKDSTemplate();
                            } else {
                                KDSWidget.dataSource.data[indexToReplace] = Util.extend(KDSWidget.dataSource.data[indexToReplace], requests[key], true);
                            }	
						} else if (~indexToAgroup && indexToAgroup != null) {
                            requests[key].isNewOrder = false;
                            if (KDSWidget.dataSource.data[indexToAgroup].NRATRAPRODITPE == requests[key].NRATRAPRODITPE) {
                                KDSWidget.dataSource.data[indexToAgroup].items = KDSWidget.dataSource.data[indexToAgroup].items.concat(requests[key].items);
                                var now = new moment();
                                KDSWidget.dataSource.data[indexToAgroup].LASTUPDATE = now.format();
                                // KDSWidget.updateKDSTemplate();
                            } else {
                                requests[key].isNewOrder = true;
                                KDSWidget.dataSource.data.push(requests[key]);
                            }
                        } else if (localEnvironment.IDHABPREENVIO || requests[key].IDLIBERADO == 'S') {
							indexToReplace = KDSWidget.dataSource.data.length;
							requests[key].isNewOrder = true;
							KDSWidget.dataSource.data.push(requests[key]);
						} else {

							indexToReplace = KDSWidget.dataSource.data.length;
							requests[key].isNewOrder = true;

						}
						// KDSWidget.updateKDSTemplate();
						if (requests[key].IDSITITPEDFOS == "C" && localEnvironment.IDHABAGRUPPROD == "N") {
							KDSWidget.dataSource.data.forEach((pedido, indexPedido) => {
								if(pedido.NRPEDIDOFOS == requests[key].NRPEDIDOFOS && pedido.NRITPEDIDOFOS == requests[key].NRITPEDIDOFOS) {
									KDSWidget.dataSource.data[indexPedido] = requests[key];
								}
							});
							let confirmCancel = self.verifyCancelOrigem(localEnvironment, requests[key]);
							if (confirmCancel == "N") {
								OrderActionService.orderOnClick(KDSWidget.dataSource.data[indexToReplace]);
							}
						} else if (requests[key].IDSITITPEDFOS == "C") {
							let confirmCancel = self.verifyCancelOrigem(localEnvironment, requests[key]);
							if (confirmCancel == "N") {
								OrderActionService.orderOnClick(KDSWidget.dataSource.data[indexToReplace]);
							}
						}
						// KDSWidget.updateKDSTemplate();
					}
				}
			});

            if(localEnvironment.IDHABEXPFLUID == 'S'){	
                let tempData = KDSWidget.dataSource.data;
    
                tempData.forEach( order  => {
                    //Verificar através do NRSEQPRODCOMKDS se estou no mesmo combo
                    let currentProm = '';
                    order.items.forEach( (item, idx) => {
    
    
                        if(!item.processed){
    
                            item.firstPromo = true;
                            item.lastPromo = false;
                            item.isPromo = false;
    
                            //Algo mudou
                            if( currentProm != '' && currentProm != item.NRSEQPRODCOMKDS){
                                if(currentProm != ''){
                                    //Insere DivisoriaInicial
                                    item.lastPromo = true;
                                }
                            }
    
                            if (item.NRSEQPRODCOMKDS){
                                item.isPromo = true;
                            }
    
                            //Atualiza currentProm
                            currentProm = item.NRSEQPRODCOMKDS;
    
                            item.DSCHAVEPEDPAI = '';
                            item.NRPRATO = item.NRSEQPRODCOMKDS;
                            item.NRSEQPRODCOMKDS = '';
                            item.NMPROMOCAO = '';
                            item.processed = true;
                        }
                    });
                });
            }
			if (localEnvironment.IDHABAGRUPPRODPOSMESA === 'S') {
				KDSWidget.orderDataSourceByPositionTable();
				changeColor();
			}else if (localEnvironment.IDORDPEDCON === 'S') {
				KDSWidget.orderDataSourceByActionLabel();
			} else {
				KDSWidget.orderDataSouce();
			}
			SummaryItemsService.updateSummaryItems();
			if (oldLength === 0 && KDSWidget.dataSource.data[0]) {
				NavigatorService.setFirstOrderAsSelected();
			}
			KDSWidget.updateKDSTemplate();
		}
	};

	this.verifyCancelOrigem = function (env, order) {
		var origem = order.IDORIGEMVENDA.slice(0, 3);
		var confirCancel = '';
		switch (origem) {
			case 'DLV':
				confirCancel = env.IDHABCANCDLV;
				break;
			case 'THR':
				confirCancel = env.IDHABCANCTHR;
				break;
			case 'TGO':
				confirCancel = env.IDHABCANCTGO;
				break;
			case 'BAL':
				confirCancel = env.IDHABCANCBAL;
				break;
			case 'CMD':
				confirCancel = env.IDHABCANCCMD;
				break;
			case 'MES':
				confirCancel = env.IDHABCANCMES;
				break;
			default:
				confirCancel = env.IDHABCANCBAL;
				break;
		}

		return confirCancel;
	};


	this.onItemProducaoConclude = function (requestkey) {
		let length = requestkey.length;
		let parseKey = requestkey.substring(length - 2, length);
		let toDeleteOrder = KDSWidget.dataSource.data.filter(function (order) {
			if (order.orderKey.substring(0, 15) == requestkey.substring(0, 15) && parseKey == '01') {
				return order.orderKey;
			}
			return order.orderKey == requestkey;
		});
		/* let toDeleteOrder = KDSWidget.dataSource.data.filter(function (order) {
			if (order.orderKey.substring(0, 15) == requestkey.substring(0, 15)) {
				return order;
			}
			return order.orderKey == requestkey;
		}); */
		/* if (toDeleteOrder) {
			let index = KDSWidget.dataSource.data.indexOf(toDeleteOrder);
			if (~index) {
				KDSWidget.dataSource.data.splice(index, 1);
				SummaryItemsService.updateSummaryItems();
			}
		} */
		if (toDeleteOrder.length && toDeleteOrder.length > 0) {
			for (let i = 0; i < toDeleteOrder.length; i++) {
				let index = KDSWidget.dataSource.data.indexOf(toDeleteOrder[i]);
				if (~index) {
					KDSWidget.dataSource.data.splice(index, 1);
				}
			}
		}
		SummaryItemsService.updateSummaryItems();
	};

	this.reloadWidget = function (widget) {
		widget.dataSource.data = [];
		widget.getWaitingItems = self.getWaitingItems;
	};

	this.getWaitingItems = function (items, widget, setWaitingItems) {
		var keys = Object.keys(items);
		if (keys.length > 0) {
			for (var key in items) {
				var waitingItems = [];
				let waitingItem = {};
				var productName = waitingItems[items[key][widget.orderItemsField][0][widget.orderItemDescField]];
				if (productName) {
					waitingItems[productName[widget.orderItemDescField]][widget.orderItemQttyField] += items[key][widget.orderItemsField][0][widget.orderItemQttyField];
				} else {
					waitingItem[widget.orderItemQttyField] = items[key][widget.orderItemsField][0][widget.orderItemQttyField];
					waitingItem[widget.orderItemDescField] = items[key][widget.orderItemsField][0][widget.orderItemDescField];
					waitingItems[waitingItem[widget.orderItemDescField]] = waitingItem;
				}
				if (key == keys[keys.length - 1]) {
					setWaitingItems(waitingItems);
				}
			}
		} else {
			KDSWidget.tratedWaitingItems = [];
		}
	};

	this.setWaitingItems = function (waitingItems) {
		var returnArray = [];
		for (var index in waitingItems) {
			returnArray.push(waitingItems[index]);
			KDSWidget.tratedWaitingItems = returnArray;
		}
	};

	this.clearRequestsScreen = function () {
		ScreenService.confirmMessage('Você tem certeza que deseja limpar todos os pedidos?',
			'question',
			function () {
				var CDFILIAL = localEnvironment.CDFILIAL;
				var CDSETOR = localEnvironment.CDSETOR;

				SocketServiceKDS.clearScreen(CDFILIAL, CDSETOR);
				ScreenService.successNotification('Operação concluida com sucesso!');
				ScreenService.toggleSideMenu();
				window.customKDSReload();
			},
			function () { }
		);
	};

	this.productionLineDeactivated = function () {
		ScreenService.confirmMessage('Deseja desativar esta linha de produção?',
			'question',
			function () {
				localEnvironment = environment.getLocalEnvironment();
				let CDSETOR = localEnvironment.CDSETOR;
				let lineProductionRoute = '/StatusLineProduction/update';

				var params = {
					requestType: "Row",
					row: {
						"CDSETOR": CDSETOR
					}
				};

				RestEngine.post(lineProductionRoute, params).then(function (response) {
				}).catch(err => {
					return err;
				})
					.finally(() => {
						window.customKDSReload();
					});
			},
			function () { }
		);
	};

	this.toggleNotInitializedProducts = function () {
		if (localEnvironment.IDTIPOSETOR === 'P') {
			localEnvironment.hasSummary = !localEnvironment.hasSummary;
			environment.setLocalEnvironment(localEnvironment);
			var widget = templateManager.container.getActiveWidget();
			this.handleNotInitializedProducts(widget);
			ScreenService.toggleSideMenu();
		} else {
			ScreenService.toggleSideMenu();
			ScreenService.alertNotification('Opção não válida para setor de expedição');
		}
	};

	this.toggleWaitingProducts = function () {
		if (localEnvironment.IDTIPOSETOR === 'P') {
			localEnvironment.hasWaiting = !localEnvironment.hasWaiting;
			environment.setLocalEnvironment(localEnvironment);
			var widget = templateManager.container.getActiveWidget();
			this.handleWaitingProducts(widget);
			ScreenService.toggleSideMenu();
		} else {
			ScreenService.toggleSideMenu();
			ScreenService.alertNotification('Opção não válida para setor de expedição');
		}
	};

	this.handleWaitingProducts = function (widget) {
		widget.hasWaiting = localEnvironment.hasWaiting;
		templateManager.updateTemplate();
	};

	this.handleNotInitializedProducts = function (widget) {
		widget.hasSummary = localEnvironment.hasSummary;
		templateManager.updateTemplate();
	};

	this.freezeUpdates = function (widget) {
		if (!KDSWidget.frozen) {
			ScreenService.confirmMessage('Você tem certeza que deseja pausar o funcionamento do KDS? (não será possivel modificar os pedidos presentes e pedidos feitos enquanto pausado serão ignorados)',
				'question',
				function () {
					SocketServiceKDS.freezeUpdates();
					ScreenService.successNotification('Operação concluida com sucesso!');
					ScreenService.toggleSideMenu();
				},
				function () { }
			);
		} else {
			SocketServiceKDS.unfreezeUpdates();
			ScreenService.successNotification('Operação concluida com sucesso!');
			ScreenService.toggleSideMenu();
		}
	};

	this.onFreezeUpdates = function (widget) {
		KDSWidget.frozen = true;
		self.reloadWidget(KDSWidget);
		ScreenMessagesService.clearMessages();
	};
	this.onUnfreezeUpdates = function (widget) {
		KDSWidget.frozen = false;
		setTimeout(function () { SocketServiceKDS.getData(environmentParams.CDFILIAL, environmentParams.CDSETOR, environmentParams.IDTIPOSETOR); }, 1000);
	};

	// statistics
	this.onStatisticsUpdate = function (data) {
		if (KDSWidget) {
			var medio = moment({
				'seconds': data.TEMPOMEDIO
			});
			var inicio = moment({
				'seconds': data.TEMPOINICIO
			});
			var atraso = moment({
				'seconds': data.TEMPOATRASO
			});
			var mediasetordia = moment({
				'seconds': parseFloat(data.MEDIAKDSDIA)
			});
			var mediasetorhora = moment({
				'seconds': parseFloat(data.MEDIAKDSHR)
			});
			var mediasetorhoraant = moment({
				'seconds': parseFloat(data.MEDIAKDSULTHR)
			});
			var mediaexpedia = moment({
				'seconds': parseFloat(data.MEDIADIA)
			});
			var mediaexpehora = moment({
				'seconds': parseFloat(data.MEDIAHORA)
			});

			var mediaexpehoraant = moment({
				'seconds': parseFloat(data.MEDIAHORAANT)
			});

			medio._isValid = true;
			KDSWidget.TEMPOMEDIO = medio.format('HH:mm:ss');
			inicio._isValid = true;
			KDSWidget.TEMPOINICIO = inicio.format('HH:mm:ss');
			atraso._isValid = true;
			KDSWidget.TEMPOATRASO = atraso.format('HH:mm:ss');
			mediasetordia._isValid = true;
			KDSWidget.MEDIAKDSDIA = mediasetordia.format('HH:mm:ss');
			mediasetorhora._isValid = true;
			KDSWidget.MEDIAKDSHR = mediasetorhora.format('HH:mm:ss');
			mediasetorhoraant._isValid = true;
			KDSWidget.MEDIAKDSULTHR = mediasetorhoraant.format('HH:mm:ss');
			mediaexpedia._isValid = true;
			KDSWidget.MEDIADIA = mediaexpedia.format('HH:mm:ss');
			mediaexpehora._isValid = true;
			KDSWidget.MEDIAHORA = mediaexpehora.format('HH:mm:ss');
			mediaexpehoraant._isValid = true;
			KDSWidget.MEDIAHORAANT = mediaexpehoraant.format('HH:mm:ss');

			KDSWidget.QTDIA = Math.trunc(data.QTDIA);
			KDSWidget.QTHORA = Math.trunc(data.QTHORA);
			KDSWidget.QTHORAANT = Math.trunc(data.QTHORAANT);
			KDSWidget.SUMQTCONCLUIDO = Math.trunc(data.SUMQTCONCLUIDO);
			KDSWidget.SUMQTCONCLUIDOHR = Math.trunc(data.SUMQTCONCLUIDOHR);
			KDSWidget.SUMQTCONCLUIDOULTHR = Math.trunc(data.SUMQTCONCLUIDOULTHR);

			var porc = parseFloat(data.PORCENTAGEM).toFixed(2).toString().replace('.', ',');
			KDSWidget.PORCENTAGEM = porc + "%";
		}
	};

	function changeColor(){
		let controle = false;
		var nrpedidofosPrimeiro = KDSWidget.dataSource.data[0].NRPEDIDOFOS;

		KDSWidget.dataSource.data.forEach((individual, index) => {
			var nrpedidofos  = individual.NRPEDIDOFOS;
			if(nrpedidofos == nrpedidofosPrimeiro){
				KDSWidget.dataSource.data[index].MUDACOR = controle;
			}else{
				nrpedidofosPrimeiro = individual.NRPEDIDOFOS;
				controle = !controle;
				KDSWidget.dataSource.data[index].MUDACOR = controle;
			}
		});
	}

	function initialize() {
		eventAggregator.subscribe('onProductionResponse', self.onItemProducaoChange);
		eventAggregator.subscribe('onProductionConclude', self.onItemProducaoConclude);
		eventAggregator.subscribe('onStatisticsUpdate', self.onStatisticsUpdate);
		eventAggregator.subscribe('onDeleteOrders', self.onItemDelete);
		eventAggregator.subscribe('onPong', SocketServiceKDS.onPong);
		eventAggregator.subscribe('onNewMessage', ScreenMessagesService.onNewMessage);
		eventAggregator.subscribe('onDeleteMessage', ScreenMessagesService.onDeleteMessage);
		eventAggregator.subscribe('onFreezeUpdates', self.onFreezeUpdates);
		eventAggregator.subscribe('onUnfreezeUpdates', self.onUnfreezeUpdates);
	}

	initialize();
}

Configuration(function (ContextRegister, eventEngine) {
	//register the event to be used with KDS
	eventEngine.registerEvent('KDSOrderClick', 'click');
	eventEngine.registerEvent('KDSOrderSelect', 'onSelect');
	eventEngine.registerEvent('KDSOrderUnselect', 'onUnselect');
	eventEngine.registerEvent('KDSOrderRollback', 'rollback');
	ContextRegister.register('DisplayController', DisplayController);
});